import React from 'react'
import ReactDOM from 'react-dom/client'
import { BrowserRouter } from 'react-router-dom'
import AppWrapper from './AppWrapper'
import { setupCacheCleanup, setupVisibilityBasedCleanup } from './utils/cache/cacheCleanup';
import { initJavaScriptOptimizations } from './utils/optimizationInitializer';
import { initializeCriticalResources } from './utils/externalResources';
import { initializeThirdPartyOptimization } from './utils/thirdPartyOptimizer';
import { bootstrapAPI, preloadCommonData } from './api/apiBootstrap';

// Font fallback styles - imported first to ensure text is visible immediately
import './styles/font-fallback.css';
// Custom font styles with proper fallbacks
import './styles/fonts.css';
// Note: The main CSS file is already loaded in index.html

// Enable debug mode in development environment
if (process.env.NODE_ENV === 'development') {
  console.log('🔍 Development mode detected - enabling debug features');
  localStorage.setItem('debug-mode', 'true');
}

// Initialize critical resources immediately for faster rendering
console.log('🔄 Initializing critical resources...');
initializeCriticalResources().catch(error => {
  console.warn('Non-critical error initializing resources:', error);
});

// Initialize third-party optimization
console.log('🔄 Initializing third-party optimization...');
initializeThirdPartyOptimization();

// Set up cache cleanup
console.log('📋 Setting up cache management...');
setupCacheCleanup(30); // Clean up expired cache items every 30 minutes
setupVisibilityBasedCleanup(); // Clean up when app becomes visible again

// Use dynamic imports for non-critical CSS to defer loading
console.log('🎨 Loading non-critical CSS...');
const loadNonCriticalCSS = () => {
  import('./styles/globals.css')
  import('./components/ui/darkTheme.css')
  import('./styles/settings-enhanced.css')
}

// Load non-critical CSS after page content is rendered
if (document.readyState === 'complete') {
  loadNonCriticalCSS()
} else {
  window.addEventListener('load', loadNonCriticalCSS)
}

// Initialize JavaScript optimizations
initJavaScriptOptimizations();

// Bootstrap the API system
console.log('🌐 Bootstrapping API subsystem...');
bootstrapAPI()
  .then(() => {
    console.log('🌐 API subsystem bootstrapped successfully');
  })
  .catch(error => {
    console.error('🚨 Error bootstrapping API subsystem:', error);
    // Continue to render the app even if API bootstrap fails
  });

// Load API testing utility in development mode
if (process.env.NODE_ENV === 'development' || localStorage.getItem('debug-mode') === 'true') {
  console.log('🧪 Loading API testing tools...');
  import('./utils/testAPI').catch(error => {
    console.warn('Non-critical error loading API testing tools:', error);
  });
}

console.log('🚀 Rendering application...');
ReactDOM.createRoot(document.getElementById('root')!).render(
  <React.StrictMode>
    <BrowserRouter>
      <AppWrapper />
    </BrowserRouter>
  </React.StrictMode>,
)

// After app is rendered and visible, preload common API data
const preloadApiData = () => {
  // Small delay to ensure UI is responsive first
  setTimeout(() => {
    preloadCommonData()
      .then(() => {
        console.log('🌐 Common API data preloaded successfully');
      })
      .catch(error => {
        console.warn('Non-critical error preloading API data:', error);
      });
  }, 1000);
};

// Register service worker after application is rendered
// This ensures it doesn't block initial rendering
console.log('🔄 Initializing service worker...');
const registerServiceWorker = async () => {
  try {
    // In development mode, skip service worker registration to avoid conflicts
    if (import.meta.env.DEV) {
      console.log('🔄 Skipping service worker registration in development mode');
      return null;
    }
    
    const { registerServiceWorker, getServiceWorkerStatus } = await import('./utils/serviceWorkerRegistration');
    const registration = await registerServiceWorker();
    
    if (registration) {
      console.log('🔄 Service worker registration successful');
      
      // Log the current service worker status
      const status = await getServiceWorkerStatus();
      console.log('🔄 Service Worker Status:', {
        supported: status.supported,
        registered: status.registered,
        controller: status.controller,
        waiting: status.waiting,
        installing: status.installing
      });
    } else {
      console.log('🔄 Service worker registration failed or not supported');
    }
  } catch (error) {
    console.warn('Non-critical error registering service worker:', error);
  }
};

// Delay service worker registration and API data preloading to improve initial load performance
if (document.readyState === 'complete') {
  // If page is already loaded, register immediately
  registerServiceWorker();
  preloadApiData();
} else {
  // Otherwise, register after page load
  window.addEventListener('load', () => {
    // Small additional delay to ensure UI is responsive first
    setTimeout(registerServiceWorker, 500);
    preloadApiData();
  });
}

// Log application version
console.log(`📱 TrackAurora v${import.meta.env.VITE_APP_VERSION || '1.0.0'} initialized`);
console.log('🌐 Cache and JavaScript optimizations enabled');
